#include <Wire.h>
#define SLAVE_ADDR 0x02   // 7-bit Slave Address

int count = 0 ;
char str[32] ;

// --- SLAVE COMMANDS
#define CMD_CLS 1         // 1 : CLS
#define CMD_SCROLL 2      // 2 : SCROLL
#define CMD_LINE 3        // 3 : LINE [XS][YS][XE][YE][COLOR]
#define CMD_RECT 4        // 4 : RECT [X][Y][W][H][COLOR]
#define CMD_FILLRECT 5    // 5 : FILLRECT [X][Y][W][H][COLOR]
#define CMD_CIRCLE 6      // 6 : CIRCLE [X][Y][R][COLOR]
#define CMD_FILLCIRCLE 7  // 7 : FILLCIRCLE [X][Y][R][COLOR]
#define CMD_LOCATE 8      // 8 : LOCATE [X] [Y]
#define CMD_FONTCOLOR 9   // 9 : FONT COLOR [COLOR]
#define CMD_PRINT 10      // A : PRINT [文字列 MAX 16]
#define CMD_PRINTLOCATE 11// B : PRINT [X][Y][文字列 MAX 16]
#define CMD_7SEGDISP 12   // C : 7SEG DISP [X][Y][N][COLOR]

#define MONOOLED_WHITE 1
#define MONOOLED_BLACK 0

// SLAVEにコマンドを送信
void oledSendCommand(uint8_t command) {
  Wire.beginTransmission(SLAVE_ADDR);
  Wire.write(command);
  Wire.endTransmission();
}

// SLAVEにコマンド+パラメタを送信
void oledSendCommand(uint8_t command,uint8_t param1,uint8_t param2=0,uint8_t param3=0,uint8_t param4=0,uint8_t param5=0) {
  Wire.beginTransmission(SLAVE_ADDR);
  Wire.write(command);
  Wire.write(param1);
  Wire.write(param2);
  Wire.write(param3);
  Wire.write(param4);
  Wire.write(param5);
  Wire.endTransmission();
}

// SLAVEにCMD_PRINTを送信
void oledSendCommand(char *str) {
  Wire.beginTransmission(SLAVE_ADDR);
  Wire.write(CMD_PRINT);
  for (int i = 0 ; i< 16 && str[i] != 0 ;i++) {
    Wire.write(str[i]);
  }
  Wire.endTransmission();
}

// SLAVEにCMD_PRINTを送信
void oledSendPrint(uint8_t param1,uint8_t param2,char *str) {
  Wire.beginTransmission(SLAVE_ADDR);
  Wire.write(CMD_PRINTLOCATE);
  Wire.write(param1);
  Wire.write(param2);
  for (int i = 0 ; i< 16 && str[i] != 0 ;i++) {
    Wire.write(str[i]);
  }
  Wire.endTransmission();
}

void setup() {
  Wire.begin();
  delay(3500) ;
  oledSendCommand(CMD_CLS) ;
}

void loop() {
  sprintf(str,"COUNT - %04d",count) ;
  oledSendPrint(0,0,str) ;
  oledSendCommand(CMD_7SEGDISP,6,4,count % 10 ,MONOOLED_WHITE) ;
  oledSendCommand(CMD_7SEGDISP,4,4,(count / 10) % 10 ,MONOOLED_WHITE) ;
  oledSendCommand(CMD_7SEGDISP,2,4,(count / 100) % 10 ,MONOOLED_WHITE) ;
  oledSendCommand(CMD_7SEGDISP,0,4,(count / 1000) % 10 ,MONOOLED_WHITE) ;

  oledSendCommand(CMD_LINE,20,17,75,31,MONOOLED_WHITE) ;
  oledSendCommand(CMD_LINE,20,31,75,17,MONOOLED_WHITE) ;
  oledSendCommand(CMD_FILLRECT,80,20,15,15,MONOOLED_WHITE) ;
  oledSendCommand(CMD_RECT,78,18 ,19,19,MONOOLED_WHITE) ;

  oledSendCommand(CMD_CIRCLE,85,50,4,MONOOLED_WHITE) ;
  oledSendCommand(CMD_CIRCLE,85,50,6,MONOOLED_WHITE) ;
  oledSendCommand(CMD_CIRCLE,85,50,8,MONOOLED_WHITE) ;

  oledSendCommand(CMD_FILLCIRCLE,105,50,8,MONOOLED_WHITE) ;
  if ((count % 2) == 0) { 
    oledSendCommand(CMD_FILLCIRCLE,105,50,4,MONOOLED_BLACK) ;
  }
  
  delay(1000) ;
  count ++ ;
  count %= 10000 ;
}
